﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Editors;
using System.Windows.Controls.Primitives;
using Infragistics.Controls.Editors.Primitives;
using System.Threading;

namespace SilverlightApplication1
{
    public partial class MainPage : UserControl
    {
        public MainWindowViewModel ViewModel { get; set; }

        MultiColumnComboItemsPanel itemsPanel;
        Border popupBorder;
        int numCellsInRow;
        double headerHeight;

        public MainPage()
        {
            ViewModel = new MainWindowViewModel();
            this.DataContext = ViewModel;

            InitializeComponent();
        }

        private void xamMultiColumnComboEditor1_Loaded(object sender, RoutedEventArgs e)
        {
            // The combo editor isn't in the visual tree yet.
            (sender as XamMultiColumnComboEditor).LayoutUpdated += xamMultiColumnComboEditor1_LayoutUpdated;
        }

        private void xamMultiColumnComboEditor1_LayoutUpdated(object sender, EventArgs e)
        {
            // Grab the popup from the visual tree.
            Grid grid = VisualTreeHelper.GetChild(xamMultiColumnComboEditor1, 0) as Grid;
            Popup popup = ((grid.Children[0] as Border).Child as Grid).Children[0] as Popup;

            // Handle the popup opened event.  The grid rows will exist by the time this event is fired.
            popup.Opened += new EventHandler(popup_Opened);

            // Dont need to handle this event anymore.
            xamMultiColumnComboEditor1.LayoutUpdated -= xamMultiColumnComboEditor1_LayoutUpdated;
        }

        void popup_Opened(object sender, EventArgs e)
        {
            Popup popup = sender as Popup;
            popupBorder = (popup.Child as System.Windows.Controls.Grid).Children[0] as Border;

            // This items panel contains all the rows, including the header, in the grid.
            itemsPanel = FindItemsPanel(popup.Child) as MultiColumnComboItemsPanel;
            itemsPanel.LayoutUpdated += new EventHandler(itemsPanel_LayoutUpdated);
        }

        void itemsPanel_LayoutUpdated(object sender, EventArgs e)
        {
            if (itemsPanel != null && itemsPanel.Children.Count != 0)
            {
                (popupBorder.Parent as Grid).InvalidateMeasure();
                (popupBorder.Parent as Grid).InvalidateArrange();

                double verticalScrollBarWidth = ((itemsPanel.Parent as Grid).Children[1] as FrameworkElement).Width;
                double horizontalScrollBarHeight = ((itemsPanel.Parent as Grid).Children[2] as FrameworkElement).Height;

                ComboCellsPanel headerPanel = null;
                ComboCellsPanel actualRowPanel = null;

                foreach (ComboCellsPanel panel in itemsPanel.Children)
                {
                    if (panel.Row is ComboHeaderRow)
                        headerPanel = panel;
                    else if (actualRowPanel == null)
                        actualRowPanel = panel;
                }

                headerHeight = headerPanel.ActualHeight;
                double actualRowWidth = 0;
                for (int i = 0; i < headerPanel.Children.Count - 1; i++)
                {
                    ComboHeaderCellControl control = headerPanel.Children[i] as ComboHeaderCellControl;
                    actualRowWidth += control.ActualWidth;
                }

                foreach (ComboCell cell in actualRowPanel.Row.Cells)
                    cell.Control.Loaded += comboCellControl_Loaded;

                // The 5 is just to push the width over enough to hide the scrollbar.
                popupBorder.Width = actualRowWidth + 5;
            }

            itemsPanel.LayoutUpdated -= itemsPanel_LayoutUpdated;
        }

        int cellLoadedCounter = 0;
        void comboCellControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Unhook from the event.
            (sender as ComboCellControl).Loaded -= comboCellControl_Loaded;

            // Increment the counter.
            cellLoadedCounter++;

            if (cellLoadedCounter >= (sender as ComboCellControl).Cell.Row.Cells.Count)
            {
                double cellHeight = (sender as ComboCellControl).Cell.Row.ActualHeight;
                double totalRowHeight = headerHeight + (cellHeight * ViewModel.Data.Count);
                popupBorder.Height = totalRowHeight;

                // Invalidate the popup border's parent measure so it renders properly.
                (popupBorder.Parent as Grid).InvalidateMeasure();
            }
        }

        private void xamMultiColumnComboEditor1_DropDownOpened(object sender, EventArgs e)
        {
            Grid grid = VisualTreeHelper.GetChild(sender as XamMultiColumnComboEditor, 0) as Grid;
            Popup popup = ((grid.Children[0] as Border).Child as Grid).Children[0] as Popup;

            ((popup.Child as System.Windows.Controls.Grid).Children[0] as Border).Width = 1000;
        }

        /// <summary>
        /// Helper method used to find the first grid row in the visual tree.
        /// </summary>
        /// <param name="root"></param>
        /// <returns></returns>
        private DependencyObject FindItemsPanel(DependencyObject root)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(root); i++)
            {
                DependencyObject dObj = VisualTreeHelper.GetChild(root, i);
                if (dObj.GetType() == typeof(MultiColumnComboItemsPanel))
                    return dObj;

                dObj = FindItemsPanel(dObj);
                if (dObj != null)
                    return dObj;
            }

            return null;
        }
    }
}
